#
# SPoTiFy Jukebox version 20210531
#
import os
import sys
import spotipy
import spotipy.util as util
import json
import sqlite3 
import traceback

IGNORE_UNICODE=1

if IGNORE_UNICODE==1:
  import unicodedata

# CONFIGURATION SECTION
username = ''
os.environ["SPOTIPY_CLIENT_ID"]=''
os.environ["SPOTIPY_CLIENT_SECRET"]='' 
os.environ["SPOTIPY_REDIRECT_URI"]='http://127.0.0.1:8421'
device=''
scope = 'user-read-playback-state,user-library-read,user-modify-playback-state'

db_location="sptfj.db"

def cfg_get_device():
  return(device)

def cfg_set_device(devid):
  global device
  device=devid

def get_max_rowid():
  try:
    con=sqlite3.connect(db_location)
    qry="select max(rowid) from sptf_search_history"
    c = con.cursor()                                                        
    c.execute(qry)
    rows = c.fetchall()                                                                                                               
    con.close()
    return (rows)
  except Exception:
    traceback.print_exc(file=sys.stdout)
    print ("ERR [ FAILED TO GET MAX ROWID ] 001000x0010 (1) : get_max_rowid")

def sptf_currently_playing():

  token = util.prompt_for_user_token(username, scope)

  artist="..."
  album="..."
  song="..."

  if token:
    sp = spotipy.Spotify(auth=token)
    res=sp.currently_playing();
    try:
      #print(res)
      if (type(res) != "None"):
        artist=res["item"]["album"]["artists"][0]["name"]
        album=res["item"]["album"]["name"]
        song=res["item"]["name"]
    except:
      pass
    return([artist,album,song])
  else:
    print("Error during action for ", username)

def get_item_type(id):

  try:
    con=sqlite3.connect(db_location)
    qry="select type from sptf_search_history where rowid = ?"
    c = con.cursor()                                                        
    c.execute(qry,(id,))                                                          
    rows = c.fetchall()                                                                                                               
    #print (rows)                                                           
    con.close()
    return (rows)
  except Exception:
    traceback.print_exc(file=sys.stdout)
    print ("ERR [ FAILED TO GET ITEM TYPE ] 001000x0020 (1) : get_item_type")

def get_items_by_range(low_id,limit,filt="%"):

  try:
    con=sqlite3.connect(db_location)
    qry="""select rowid, res
      from sptf_search_history 
      where rowid >= ? 
      and lower(res) like ?
      order by rowid asc
      limit ?"""
    c = con.cursor()                                                        
    c.execute(qry,(low_id,"%"+filt+"%",limit,))
    rows = c.fetchall()
    #print (rows)                                                           
    con.close()
    return (rows)
  except Exception:
    traceback.print_exc(file=sys.stdout)
    print ("ERR [ FAILED TO GET SEARCH ] 001000x0020 (1) : get_search")

def get_search(id):
  try:
    con=sqlite3.connect(db_location)
    qry="select needle, res from sptf_search_history where rowid = ?"
    c = con.cursor()                                                        
    c.execute(qry,(id,))                                                          
    rows = c.fetchall()                                                                                                               
    #print (rows)                                                           
    con.close()
    return (rows)
  except Exception:
    traceback.print_exc(file=sys.stdout)
    print ("ERR [ FAILED TO GET SEARCH ] 001000x0020 (1) : get_search")

def sptf_get_devices():

  token = util.prompt_for_user_token(username, scope)

  devices=[]  
  if token:
    sp=spotipy.Spotify(auth=token)
    dev=sp.devices();
    #print("ID"+"|"+"Active"+"|"+"Name"+"|"+"Type")
    for d in dev["devices"]:
      devices.append([d["id"],d["is_active"],d["name"],d["type"]])
      #print(d["id"]+"|"+str(d["is_active"])+"|"+d["name"]+"|"+d["type"])
    return(devices)
  else:
    return(None)

def sptf_display_devices():

  token = util.prompt_for_user_token(username, scope)
  
  if token:
      print("Put desired ID from here into sptf.py to device variable")
      sp = spotipy.Spotify(auth=token)
      dev=sp.devices();
      #print(dev["devices"])
      print("ID"+"|"+"Active"+"|"+"Name"+"|"+"Type")
      for d in dev["devices"]:
        print(d["id"]+"|"+str(d["is_active"])+"|"+d["name"]+"|"+d["type"])
  else:
      print("Error during action for ", username)

def sptf_play_playlist(ctx_uri):

  token = util.prompt_for_user_token(username, scope)
  
  if token:
      sp = spotipy.Spotify(auth=token)
      sp.start_playback(device,context_uri=ctx_uri);
      return("Action: Pause, Result: OK")
  else:
      return("Error during action for ", username)

def sptf_play_ctx(ctx_uri):

  token = util.prompt_for_user_token(username, scope)
  
  if token:
      sp = spotipy.Spotify(auth=token)
      sp.start_playback(device,context_uri=ctx_uri);
      return("Action: Pause, Result: OK")
  else:
      return("Error during action for ", username)

def init_db():
  try:
    conn=sqlite3.connect(db_location)
    cur = conn.cursor()
    cur.execute("""create table if not exists sptf_search_history 
      (needle text, res text, type text)""")
  except Exception:
    traceback.print_exc(file=sys.stdout)
    print ("ERR [ FAILED TO INIT DB ] 001000x0030 (1) : init_db")

def delete_record(rowid):

  try:
    conn=sqlite3.connect(db_location)
    cur = conn.cursor()
    cur.execute("""create table sptf_search_history_tmp as 
      select * 
      from sptf_search_history 
      where rowid != ?""", (rowid,))
    cur.execute("""drop table sptf_search_history""")
    cur.execute("""create table sptf_search_history as 
      select * 
      from sptf_search_history_tmp""")
    cur.execute("""drop table sptf_search_history_tmp""")
    conn.commit()
    conn.close()
  except Exception:
    traceback.print_exc(file=sys.stdout)
    print ("ERR [ DELETE RECORD FAILED ] 001000x0040 (1) : delete_record")

def save_search(needle,res,album_type):

  try:
    conn=sqlite3.connect(db_location)
    cur = conn.cursor()
    cur.execute("""insert into sptf_search_history 
      select ?, ?, ? 
      where not exists (
        select 1 from sptf_search_history
        where res = ?)
    """,(needle,res,album_type,res,))
    conn.commit()
    conn.close()
  except Exception:
    traceback.print_exc(file=sys.stdout)
    print ("ERR [ SAVE SEARCH FAILED ] 001000x0050 (1) : save_search")


def sptf_search_show(needle):

  token = util.prompt_for_user_token(username, scope)
  
  if token:
    sp = spotipy.Spotify(auth=token)
    dev=sp.search(needle,type='show');
    return(dev)
  else:
    print("Error during action for ", username)

def sptf_search_playlist(needle):

  token = util.prompt_for_user_token(username, scope)
  
  if token:
      sp = spotipy.Spotify(auth=token)
      dev=sp.search(needle,type='playlist');
      return(dev)
  else:
      print("Error during action for ", username)

def sptf_search(needle):

  token = util.prompt_for_user_token(username, scope)
  
  if token:
      sp = spotipy.Spotify(auth=token)
      dev=sp.search(needle);
      return(dev)
  else:
      print("Error during action for ", username)

def sptf_previous_track():

  token = util.prompt_for_user_token(username, scope)
  
  if token:
    sp = spotipy.Spotify(auth=token)
    try:
      sp.previous_track();
    except:
      return(-1)
  else:
    print("Error during action for ", username)
  return(0)

def sptf_next_track():

  token = util.prompt_for_user_token(username, scope)
  
  if token:
    sp = spotipy.Spotify(auth=token)
    try:
      sp.next_track();
    except:
      return(-1)
  else:
    print("Error during action for ", username)
  return(0)

def sptf_search_album(needle,save_to_history=1,item_type="album"):
  albums=[]
  albums_display=[]
  album=""
  song=""
  artist=""
  album_display=""
  play_id=""
  if (item_type=="album"):
    res=sptf_search(needle)
    if (len(res["tracks"]["items"])>0):
      i=res["tracks"]["items"][0]
      artist=i["album"]["artists"][0]["name"]
      song=str(i["album"]["artists"][0]["name"])
      album=i["album"]["name"]
      albums.append(str(i["album"]["uri"]))
      album_display=artist+"|"+album
      albums_display.append(album_display)
      play_id=albums[0]
  elif (item_type=="playlist"):
    res=sptf_search_playlist(needle)
    #print(res)
    if res["playlists"]["items"]!=[]:
      play_id=res["playlists"]["items"][0]["uri"]
      owner_name=res["playlists"]["items"][0]["owner"]["display_name"]
      playlist_name=res["playlists"]["items"][0]["name"]
      album_display=owner_name+"|"+playlist_name
  elif (item_type=="show"):
    res=sptf_search_show(needle)
    #print(res)
    if (res["shows"]["items"]!=[]):
      play_id=res["shows"]["items"][0]["uri"]
      publisher_name=res["shows"]["items"][0]["publisher"]
      playlist_name=res["shows"]["items"][0]["name"]
      album_display=publisher_name+"|"+playlist_name
  else:
    return(["ERR","Invalid search method"])
  try:
    sptf_play_ctx(play_id)
  except Exception as e:
    return(["ERR",str(e)])
  
  if (save_to_history==1):
    save_search(needle,album_display,item_type)
    return(["PLA",album_display])
  else:
    return(["PLA",album_display])

def uni_strip_accents(s):
  if IGNORE_UNICODE==1:
    return ''.join(c for c in unicodedata.normalize('NFD', s)
      if unicodedata.category(c) != 'Mn')
  else:
    return(s)
