#
# SPoTiFy Jukebox version 20210531
#
import curses                                                                   
import json
import locale  
import os
import sys
import traceback                                                       
from random import uniform
from sptf import cfg_get_device
from sptf import cfg_set_device
from sptf import delete_record
from sptf import get_item_type
from sptf import get_items_by_range
from sptf import get_search
from sptf import get_max_rowid
from sptf import init_db
from sptf import sptf_currently_playing
from sptf import sptf_display_devices
from sptf import sptf_get_devices
from sptf import sptf_next_track
from sptf import sptf_previous_track
from sptf import sptf_play_ctx
from sptf import sptf_search_album
from sptf import uni_strip_accents

import unicodedata


REFRESH_INTERVAL=10000 # in miliseconds 10 seconds

LANG="EN"

msg_str={}

msg_str["APP_NAME_EN"]="+ SPoTiFy Jukebox +"
msg_str["ENTRY_DIAL_EN"]="We shall play: "
msg_str["NOW_PLAYING_EN"]="Now playing: "

pos={}

pos["AN"] = 0 # APP_NAME
pos["ED"] = 2 # ENTRY_DIAL
pos["IS"] = 2 # INPUT STRING
pos["ISC"]=15 # INPUT STRING COLUMN
pos["NP"] = 4 # NOW PLAYING
pos["ER"] = 5 # NOW PLAYING
pos["SH"] = 6 # SEARCH HISTORY 
pos["HI"] = 8 # HISTORY TOP

inpstr=""

if len(sys.argv)>=2:
  if (sys.argv[1]=="--devices"):
    sptf_display_devices()
    exit(0)

devices=[]

init_db()
scr = curses.initscr()
curses.noecho()                                                               
curses.cbreak()                                                               
curses.start_color()                                                          
scr.keypad(1)                                                              
curses.init_pair(1,curses.COLOR_CYAN,curses.COLOR_BLACK) 
hiTe=curses.color_pair(1)|curses.A_BOLD
miTe=curses.color_pair(1)
loTe=curses.color_pair(1)|curses.A_DIM
scr.clear()
scr.refresh()
height,width=scr.getmaxyx()
win = curses.newwin(height,width,0,0)
if REFRESH_INTERVAL>0:
  win.timeout(REFRESH_INTERVAL)
x=0
cnt=0
last_search=""
err=""
action=""
hist_search=""
cur_hist_id=1
save_flag=1
xrowid=1
track_filter=""
playlist=""
show_name=""
new_device=-1
while x != 27: # Until ESC keypress 
  # DRAW THE TEXT USER INTERFACE
  height,width=scr.getmaxyx()
  # BANNER SECTION
  devs=sptf_get_devices()
  win.addstr(0,0," "*(width-1))
  banner=msg_str["APP_NAME_"+LANG]
  win.addstr(0,pos["AN"]," "*(width))
  devstr=" @ ["+str(len(devs))+"]: "
  win.addstr(pos["AN"],0,banner,loTe)
  devices=[]
  devices=devs
  cfg_dev_id=cfg_get_device()
  # Look if the configured device is connected if it is display it
  cfg_dev_avail=0
  did=1
  if (len(devs)>0):
    for d in devs:
      if d[0]==cfg_dev_id:
        win.addstr(0,len(banner),devstr,loTe)
        win.addstr(0,len(banner)+len(devstr),str(did)+" "+d[2]+" "+d[3],miTe)
        cfg_dev_avail=1
      did+=1
  # If configured device not availabe display first available
  did=1
  if len(devs)>0 and cfg_dev_avail==0:
    for d in devs:
      win.addstr(0,len(banner),devstr,loTe)
      win.addstr(0,len(banner)+len(devstr),str(did)+" "+d[2]+" "+d[3],loTe)
      cfg_dev_avail=1
      did+=1
      break
  # No devices available at all
  if (len(devs)==0):
    win.addstr(0,len(banner)," No Device available",loTe)
  # ORNAMENT SECTION
  #win.addstr(1,width-1,"."*(width-1),loTe)
  win.addstr(pos["ED"],0,msg_str["ENTRY_DIAL_"+LANG],loTe)
  win.addstr(pos["IS"],pos["ISC"],inpstr,hiTe)
  # NOW PLAYING SECTION
  win.addstr(pos["NP"],0,msg_str["NOW_PLAYING_"+LANG],loTe)
  curplay=sptf_currently_playing()
  artist=curplay[0] 
  artist_len=len(artist)
  album=curplay[1] 
  album_len=len(album)
  song=curplay[2] 
  song_len=len(song)
  song_row=pos["NP"]
  song_col=0
  win.addstr(pos["NP"],0," "*(width-1))
  if (artist_len>=width-1):
    artist="..."
  win.addstr(pos["NP"],0,artist+"|",hiTe)
  if (artist_len+album_len>=width-1):
    album="..."
  win.addstr(pos["NP"],len(artist)+1,album+"|",miTe)
  song_col=len(artist)+len(album)+2
  if (artist_len+album_len+song_len>=width-1):
    song=song[:width-1]
    song_col=0
    song_row=9
  #elif (artist_len+album_len+song_len>=width-1):
  #  song="..."
  win.addstr(song_row,song_col,song,loTe)
  # ACTION DISPLAY SECTION
  win.addstr(height-1,0," "*(width-1))
  if (action!="NOOP"):
    win.addstr(height-1,0,": "+action,loTe)
  # DRAW THE SEARCH HISTORY
  xrowid=get_max_rowid()[0][0]
  win.addstr(pos["SH"],0,"| History: |-[ ",loTe)
  win.addstr(pos["SH"],15,"{:05d}".format(cur_hist_id),hiTe)
  win.addstr(pos["SH"],20," / ",miTe)
  if xrowid:
    win.addstr(pos["SH"],23,"{:05d}".format(xrowid)+" ]-",loTe)
  #x=int(height/2)-1
  x=height-pos["HI"]-2
  rownum=0
  allnum=0
  if (str(xrowid) != "None" and str(xrowid) != "NoneType"):
    hist_cur=""
    #first_displayed_track=-1
    # CLEAR THE DISPLAY OF SEARCH ITEMS
    for y in range(0,x):
      win.addstr(pos["HI"]+y,0," "*(width-1))
    # LOOP FROM FIRST TO MAX ITEMS 
    lb=cur_hist_id # LOWER BOUND
    ub=x # UPPER BOUND
    # PREPARE FILTER
    if (track_filter!=""):
      track_filter=uni_strip_accents(track_filter).lower()
      lb=1
    # GET ITEMS FROM DB
    items=get_items_by_range(lb,ub,track_filter)
    # DISPLAY ITEMS
    for y in range(0,len(items)):
      it_id=items[y][0]
      it_str=items[y][1].replace('\n','')
      if (cur_hist_id==it_id):
        win.addstr(pos["HI"]+rownum,0,"> "+str(it_id)+"|"+it_str,hiTe)
      else:
        win.addstr(pos["HI"]+rownum,0,"  "+str(it_id)+"|"+it_str,loTe)
      rownum+=1
      #if (rownum>=x):
    if (track_filter!="" and rownum >= 1):
      cur_hist_id=items[0][0]
  # PROCESS ERROR MESSAGES
  if err:
    if err.find("Device not found") != -1:
      err="Device not found"
    elif err.find("list index out of range") != -1:
      err="Cannot find album"
    elif err.find("Invalid context uri") != -1:
      err="Cannot find item"
    else:
      err="Other error"
    win.addstr(pos["ER"],0,"Message: "+err,loTe)
    action="ERROR"
  else:
    win.addstr(pos["ER"],0," "*(width-1))
  #else:
  # CURSOR CLEANUP
  #x = scr.getch()
  action="NOOP"
  win.move(0, width-1) # MOVE CUROSOR
  x = win.getch()
  # HANDLE RESIZING
  if (x==curses.KEY_RESIZE): # if resize is True:
    height,width=scr.getmaxyx()
    win = curses.newwin(height,width,0,0)
    win.clear()
    win.refresh()
  win.move(0, width-1) # MOVE CUROSOR
  # PROCESS KEY PRESS
  if (x==curses.KEY_BACKSPACE or x==127): # BACKSPACE
    win.addstr(pos["IS"],pos["ISC"],str(" "*(len(inpstr)+1)))
    inpstr=inpstr[:-1]
    action="NOOP"
  elif (x==10): # ENTER
    action="SEARCH"
    if (cnt>0):
      save_flag=1
    elif (track_filter!=""):
      save_flag=0
    cnt=0
    #win.clear()
  elif (x >= 32 and x <= 126): # ANY OTHER KEY
    inpstr+=chr(x)
    cnt+=1
  else: # ANY OTHER KEY
    if type(x) != "None" and action != "NOOP":
      #action="ANY KEY ... " + chr(x) + " " + str(x) # DEBUG
      action="ANY KEY ... "
  # PROCESS ACTIONS
  if (inpstr==".k"): # MOVE TO PREVIOUS ITEM
    action="PREVIOUS ITEM       "
    if (cur_hist_id-1>=1):
      win.addstr(3,0,str(" "*(len(inpstr)+1)))
      cur_hist_id-=1
    inpstr=""
    save_flag=0
    #win.clear()
  # TMP
  elif (inpstr==".m"): # MOVE TO NEXT ITEM
    sptf_play_ctx("spotify:show:0aP2XgkWW2yJKBB8DvmWjO")
    inpstr=""
    action="NOOP"
  # TMP
  elif (inpstr==".j"): # MOVE TO NEXT ITEM
    action="NEXT ITEM         "
    if (cur_hist_id+1<=xrowid):
      win.addstr(3,0,str(" "*(len(inpstr)+1)))
      cur_hist_id+=1
    inpstr=""
    save_flag=0
    #win.clear()
  elif (inpstr[0:3]==".on"): # PLAY ON DEVICE
    try:
      new_device=int(inpstr[4:])
    except:
      pass
    #win.clear()
    save_flag=0
    ACTION="NOOP"
  elif (inpstr[0:2]==".i"): # SEARCH FOR PLAYLIST
    playlist=inpstr[3:]
    #win.clear()
    save_flag=1
    ACTION="NOOP"
  elif (inpstr[0:2]==".w"): # SEARCH FOR SHOW
    show_name=inpstr[3:]
    #win.clear()
    save_flag=1
    ACTION="NOOP"
  elif (inpstr[0:2]==".s"): # SEARCH IN ITEMS
    track_filter=inpstr[3:]
    #win.clear()
    save_flag=0
    ACTION="NOOP"
  elif (inpstr==".c"): # PLAY CURRENT TRACK
    action="CURRENT SEARCH"
    win.addstr(3,0,str(" "*(len(inpstr)+1)))
    hist_search=str(get_search(cur_hist_id)[0][0])
    action="SEARCH"
    inpstr=hist_search
    save_flag=0
  elif (inpstr==".tn"): # NEXT TRACK
    ret=sptf_next_track()
    if ret != 0:
      scr.clear()
      scr.refresh()
    inpstr=""
    action="NEXT TRACK"
  elif (inpstr==".tp"): # PREVIOUS TRACK
    ret=sptf_previous_track()
    if ret != 0:
      scr.clear()
      scr.refresh()
    inpstr=""
    action="PREVIOUS TRACK"
  elif (inpstr==".p"): # PREVIOUS ITEM
    action="PREVIOUS ITEM"
    win.addstr(3,0,str(" "*(len(inpstr)+1)))
    if (cur_hist_id-1>=1):
      cur_hist_id-=1
      hist_search=str(get_search(cur_hist_id)[0][0])
      action="SEARCH"
      inpstr=hist_search
      save_flag=0
    else:
      inpstr=""
    save_flag=0
  elif (inpstr==".n"): # NEXT ITEM
    action="NEXT ITEM"
    if (cur_hist_id+1<=xrowid):
      win.addstr(3,0,str(" "*(len(inpstr)+1)))
      cur_hist_id+=1
      hist_search=str(get_search(cur_hist_id)[0][0])
      action="SEARCH"
      inpstr=hist_search
    else:
      inpstr=""
    save_flag=0
  elif (inpstr==".l"): # LAST ITEM IN HISTORY
    cur_hist_id=xrowid
    inpstr=""
    #win.clear()
    action="LAST ITEM"
  elif (inpstr==".f"): # FIRST ITEM IN HISTORY
    cur_hist_id=1
    inpstr=""
    #win.clear()
    action="FIRST ITEM"
  elif (inpstr[0:2]==".d" and action=="SEARCH"): # Jump to history
    action="DELETE FROM PLAYLIST"
    rid=inpstr[3:]
    if (len(inpstr[3:])>0):
      rid=int(inpstr[3:])
      delete_record(rid)   
      action="DELETE FROM PLAYLIST "+str(rid)
    save_flag=0  
    cur_hist_id-=1
    inpstr=""
  elif (inpstr[0:2]==".g" and action=="SEARCH"): # Jump to history
    action="MOVE TO HISTORY "
    cur_hist_id=inpstr[3:]
    if (len(inpstr[3:])>0):
      cur_hist_id=int(inpstr[3:])
    inpstr=""
    save_flag=0  
  elif (inpstr[0:2]==".h" and action=="SEARCH"): # Jump to history
    action="GO TO HISTORY "
    #if (cur_hist_id+1<=xrowid):
    cur_hist_id=inpstr[3:]
    #win.addstr(20,0,str(cur_hist_id))
    if (len(inpstr[3:])>0):
      try:
        cur_hist_id=int(inpstr[3:])
      except:
        cur_hist_id=1
      #print(str(cur_hist_id))
    else:
      cur_hist_id=xrowid
    if cur_hist_id > xrowid:
      cur_hist_id = xrowid
    elif cur_hist_id < 1:
      cur_hist_id = 1
    hist_search=str(get_search(cur_hist_id)[0][0])
    action="SEARCH"
    inpstr=hist_search
    save_flag=0  
  elif (inpstr==".r"): # GO TO RANDOM ALBUM
    new_id=cur_hist_id
    while cur_hist_id == new_id:
      new_id = int(uniform(1,xrowid))
    cur_hist_id = new_id
    inpstr=""
    #hist_search=str(get_search(cur_hist_id)[0][0])
    #inpstr=hist_search
    action="RANDOM ITEM"
    save_flag=0
  elif (inpstr==".q"): # QUIT
    break
  # Process commands arguments
  # RESET THE SEARCH FILTER
  if (inpstr[0:2]==".s" and action=="SEARCH" and len(inpstr)==2):
    track_filter=""
    inpstr=""
    save_flag=0  
    action="CLEAR TRACK FILTER"
    cur_hist_id=1
  # PRESSED ENTER AND THERE IS SEARCH ARGUMENT
  if (action=="SEARCH" and track_filter !="" and inpstr!=""):
    if (cur_hist_id>xrowid):
      cur_hist_id=xrowid
    if (cur_hist_id>1):
      hist_search=str(get_search(cur_hist_id)[0][0])
      inpstr=hist_search
    else:
      inpstr=""
      track_filter=""
    ACTION="SEARCH HISTORY FILTER"
  # PRESSED ENTER AND THERE IS NO TRACK FILTER NOR INPUT STRING
  if (action=="SEARCH" and track_filter =="" and inpstr==""):
    if (cur_hist_id>xrowid):
      cur_hist_id=xrowid
    hist_search=str(get_search(cur_hist_id)[0][0])
    inpstr=hist_search
    #inpstr=""
    ACTION="SEARCH HISTORY NO FILTER"
  # PLAY ON DEVICE
  if (inpstr[0:3]==".on" and action=="SEARCH" and len(inpstr)>4):
    new_device-=1
    if (new_device<len(devices)):
      cfg_set_device(devices[new_device][0])
      action="PLAY ON DEVICE "+devices[new_device][2]+" "+str(new_device)
    else:
      action="DEVICE NOT CHANGED"
    inpstr=""
  # SEARCH FOR THE ITEM ON SPTF
  if (action=="SEARCH"):
    if (show_name=="" and playlist==""):
      itp=get_item_type(cur_hist_id)
      #print(itp,inpstr);
      if itp:
        if (itp[0][0]=="playlist"): 
          playlist=inpstr
        elif (itp[0][0]=="show"): 
          show_name=inpstr
    if (show_name!=""):
      inpstr=show_name
      ret=sptf_search_album(inpstr,save_flag,item_type="show")
    elif (playlist!=""):
      inpstr=playlist
      ret=sptf_search_album(inpstr,save_flag,item_type="playlist")
    else:
      ret=sptf_search_album(inpstr,save_flag,item_type="album")
    if (ret[0]=="ERR"):
      win.clear()
      err=ret[1]
    else:
      last_search=ret[1] 
      err=""
    win.addstr(3,0,str(" "*(len(inpstr)+1)))
    inpstr=""
    action=""
    playlist=""
    show_name=""
    # clear error line
    win.addstr(pos["ER"],0," "*(height-1))
  if action!="NOOP":
    #win.refresh()
    #win.clear()
    win.addstr(pos["IS"],0,str(" "*(width-1)))
    win.move(0, width-1)
  win.move(0, width-1)
# RESET THE TERMINAL TO ORIGINAL STATE
curses.nocbreak()
scr.keypad(0)
curses.echo()
curses.endwin()
